@@ -431,6 +431,13 @@ FTPQueryFeatures(const FTPCIPtr cip)
 	if (strcmp(cip->magic, kLibraryMagic))
 		return (kErrBadMagic);
 
+	if (cip->serverType == kServerTypeMicrosoftFTP) {
+		cip->hasNLST_a = kCommandNotAvailable;
+		cip->hasNLST_d = kCommandNotAvailable;
+		cip->hasSITE_UTIME = kCommandNotAvailable;
+		cip->hasRETR_tar = kCommandNotAvailable;
+	}
+
 	if (cip->serverType == kServerTypeNetWareFTP) {
 		/* NetWare 5.00 server freaks out when
 		 * you give it a command it doesn't
@@ -447,6 +454,7 @@ FTPQueryFeatures(const FTPCIPtr cip)
 		cip->hasMLSD = kCommandNotAvailable;
 		cip->hasSITE_UTIME = kCommandNotAvailable;
 		cip->hasHELP_SITE = kCommandNotAvailable;
+		cip->hasRETR_tar = kCommandNotAvailable;
 		return (kNoErr);
 	}
 
@@ -467,6 +475,7 @@ FTPQueryFeatures(const FTPCIPtr cip)
 		cip->hasHELP_SITE = kCommandNotAvailable;
 		cip->hasMFMT = kCommandNotAvailable;
 		cip->hasMFF = kCommandNotAvailable;
+		cip->hasRETR_tar = kCommandNotAvailable;
 	}
 	
 	rp = InitResponse();
@@ -520,7 +529,7 @@ FTPQueryFeatures(const FTPCIPtr cip)
 				 * (c) A protocol violation.
 				 */
 				cp = lp->line;
-				while ((*cp != '\0') && (isspace(*cp)))
+				while ((*cp != '\0') && (isspace((int) *cp)))
 					cp++;
 				if (*cp == '\0')
 					continue;
@@ -1027,6 +1036,7 @@ FTPInitConnectionInfo2(const FTPLIPtr lip, const FTPCIPtr cip, char *const buf,
 	cip->hasSITE_SBUFSZ = kCommandAvailabilityUnknown;
 	cip->STATfileParamWorks = kCommandAvailabilityUnknown;
 	cip->NLSTfileParamWorks = kCommandAvailabilityUnknown;
+	cip->hasRETR_tar = kCommandAvailabilityUnknown;
 	cip->firewallType = kFirewallNotInUse;
 	cip->startingWorkingDirectory = NULL;
 	cip->shutdownUnusedSideOfSockets = 0;
