@@ -504,21 +504,24 @@ UnLslRLine(	char *const line,
 		ftm.tm_isdst = -1;
 		if (year == 0) {
 			/* We guess the year, based on what the
-			 * current year is.  We know the file
-			 * on the remote server is either less
-			 * than six months old or less than
-			 * one hour into the future.
+			 * current year is.
+			 *
+			 * UNIX ls would always print the year
+			 * if the file was older than 6 months,
+			 * but many non-UNIX based servers do
+			 * not honor that convention, so we
+			 * now assume that if the year was not
+			 * given it implies the current year,
+			 * unless the file is up to two days 
+			 * into the future.
 			 */
 			ftm.tm_year = thisyear - 1900;
 			*ftime = mktime(&ftm);
 			if (*ftime == (time_t) -1) {
 				/* panic */
-			} else if (*ftime > (now + (15552000L + 86400L))) {
+			} else if (*ftime > (now + (2 * 86400L))) {
 				--ftm.tm_year;
 				*ftime = mktime(&ftm);
-			} else if (*ftime < (now - (15552000L + 86400L))) {
-				++ftm.tm_year;
-				*ftime = mktime(&ftm);
 			}
 		} else {
 			ftm.tm_year = year - 1900;
