@@ -14,11 +14,13 @@
 
 #if (defined(WIN32) || defined(_WINDOWS)) && !defined(__CYGWIN__)
 #	include "..\ncftp\util.h"
+#	include "..\ncftp\bookmark.h"
 #	include "..\ncftp\spool.h"
 #	include "..\ncftp\pref.h"
 #	include "..\ncftp\gl_getline.h"
 #else
 #	include "../ncftp/util.h"
+#	include "../ncftp/bookmark.h"
 #	include "../ncftp/spool.h"
 #	include "../ncftp/pref.h"
 #	include "../ncftp/gl_getline.h"
@@ -38,6 +40,7 @@ extern unsigned int gFirewallPort;
 extern char gFirewallExceptionList[256];
 extern int gFwDataPortMode;
 extern const char gOS[], gVersion[];
+extern Bookmark gBm;
 
 static void
 Usage(void)
@@ -47,7 +50,8 @@ Usage(void)
 	fp = OpenPager();
 	(void) fprintf(fp, "NcFTPGet %.5s\n\n", gVersion + 11);
 	(void) fprintf(fp, "Usages:\n");
-	(void) fprintf(fp, "  ncftpget [flags] remote-host local-dir remote-path-names...      (mode 1)\n");
+	(void) fprintf(fp, "  ncftpget [flags] remote-host local-dir remote-path-names...      (mode 1a)\n");
+	(void) fprintf(fp, "  ncftpget [flags] bookmark-name local-dir remote-path-names...    (mode 1b)\n");
 	(void) fprintf(fp, "  ncftpget -f login.cfg [flags] local-dir remote-path-names...     (mode 2)\n");
 	(void) fprintf(fp, "  ncftpget [flags] ftp://url.style.host/path/name                  (mode 3)\n");
 	(void) fprintf(fp, "  ncftpget -c [flags] remote-host remote-path-name > stdout        (mode 4)\n");
@@ -62,7 +66,8 @@ Usage(void)
 	(void) fprintf(fp, "\
   -t XX  Timeout after XX seconds.\n\
   -v/-V  Do (do not) use progress meters.\n\
-  -f XX  Read the file XX for host, user, and password information.\n\
+  -f XX  Read the file XX for host, user, and password information;\n\
+         If file XX does not exist, check for bookmark XX in $HOME/.ncftp/bookmarks.\n\
   -h XX  Connect to host XX.  Useful for overriding host in -f config.file.\n\
   -c     Read from remote host and write locally to stdout.\n\
   -C     Read from remote host and write locally to specified file.\n\
@@ -190,6 +195,7 @@ main(int argc, char **argv)
 	const char *urldirp;
 	const char *preferredLocalAddrStr = NULL;
 	char precmd[320], postcmd[320], perfilecmd[320];
+	int configLoaded = kConfigNotLoaded, configLoaded2 = kConfigNotLoaded;
 	GetoptInfo opt;
 
 	InitWinsock();
@@ -251,30 +257,11 @@ main(int argc, char **argv)
 	GetoptReset(&opt);
 	while ((c = Getopt(&opt, argc, argv, "P:u:j:p:h:e:d:t:aRTr:vVf:o:ADzZEFbcCB:W:X:Y:I:")) > 0) switch(c) {
 		case 'P':
-			fi.port = atoi(opt.arg);	
-			break;
 		case 'u':
-			(void) STRNCPY(fi.user, opt.arg);
-			memset(opt.arg, 0, strlen(fi.user));
-			opt.arg[0] = '?';
-			break;
 		case 'j':
-			(void) STRNCPY(fi.acct, opt.arg);
-			memset(opt.arg, 0, strlen(fi.acct));
-			opt.arg[0] = '?';
-			break;
 		case 'p':
-			(void) STRNCPY(fi.pass, opt.arg);	/* Don't recommend doing this! */
-			fi.leavePass = 1;
-			if (fi.pass[0] == '\0')
-				fi.passIsEmpty = 1;
-			memset(opt.arg, 0, strlen(fi.pass));
-			opt.arg[0] = '?';
-			break;
 		case 'h':
-			(void) STRNCPY(fi.host, opt.arg);
-			memset(opt.arg, 0, strlen(fi.user));
-			opt.arg[0] = '?';
+			/* Use these options later */
 			break;
 		case 'e':
 			if (strcmp(opt.arg, "stdout") == 0)
@@ -326,7 +313,10 @@ main(int argc, char **argv)
 			progmeters = 0;
 			break;
 		case 'f':
-			ReadConfigFile(opt.arg, &fi);
+			if ((configLoaded = ReadConfigFile(opt.arg, &fi)) == kErrorLoadingConfig) {
+				perror(opt.arg);
+				exit(kExitBadConfigFile);
+			}
 			break;
 		case 'o':
 			fi.manualOverrideFeatures = opt.arg;
@@ -427,6 +417,11 @@ main(int argc, char **argv)
 				dstdir = NULL;
 				flist = (const char **) argv + opt.ind + 1;
 			}
+			if (strchr(fi.host, '.') == NULL) {
+				/* Check if host argument corresponds to config file or entry from ~/.ncftp/bookmarks. */
+
+				configLoaded2 = ReadConfigFile(fi.host, &fi);
+			}
 		} else {
 			/* URL okay */
 			flist = NULL;
@@ -482,6 +477,39 @@ main(int argc, char **argv)
 		}
 	}
 
+	/* In case we loaded a bookmark or config file, we allow the following
+	 * options to override those settings.
+	 */
+	GetoptReset(&opt);
+	while ((c = Getopt(&opt, argc, argv, "P:u:j:p:h:e:d:t:aRTr:vVf:o:ADzZEFbcCB:W:X:Y:I:")) > 0) switch(c) {
+		case 'P':
+			fi.port = atoi(opt.arg);	
+			break;
+		case 'u':
+			(void) STRNCPY(fi.user, opt.arg);
+			memset(opt.arg, 0, strlen(fi.user));
+			opt.arg[0] = '?';
+			break;
+		case 'j':
+			(void) STRNCPY(fi.acct, opt.arg);
+			memset(opt.arg, 0, strlen(fi.acct));
+			opt.arg[0] = '?';
+			break;
+		case 'p':
+			(void) STRNCPY(fi.pass, opt.arg);	/* Don't recommend doing this! */
+			fi.leavePass = 1;
+			if (fi.pass[0] == '\0')
+				fi.passIsEmpty = 1;
+			memset(opt.arg, 0, strlen(fi.pass));
+			opt.arg[0] = '?';
+			break;
+		case 'h':
+			(void) STRNCPY(fi.host, opt.arg);
+			memset(opt.arg, 0, strlen(fi.user));
+			opt.arg[0] = '?';
+			break;
+	}
+
 	if (strcmp(fi.user, "anonymous") && strcmp(fi.user, "ftp")) {
 		if ((fi.pass[0] == '\0') && (fi.passIsEmpty == 0)) {
 			(void) gl_getpass("Password: ", fi.pass, sizeof(fi.pass));
@@ -617,6 +645,13 @@ main(int argc, char **argv)
 
 	(void) AdditionalCmd(&fi, precmd, NULL);
 
+	if ((configLoaded2 == kLoadedFromBookmark) && (strcmp(gBm.dir, "/") != 0) && (strcmp(gBm.dir, ".") != 0)) {
+		if ((result = FTPChdir(&fi, gBm.dir)) < 0) {
+			FTPPerror(&fi, result, kErrCWDFailed, "Warning: Could not change to bookmark's remote directory of", gBm.dir);
+		}
+
+	}
+
 	if (flist == NULL) {
 		/* URL mode */
 		es = kExitChdirTimedOut;
@@ -636,6 +671,8 @@ main(int argc, char **argv)
 			} else {
 				FTPPerror(&fi, rc, kErrCouldNotStartDataTransfer, "ncftpget", NULL);
 				es = kExitXferFailed;
+				if (fi.errNo == kErrGlobNoMatch)
+					es = kExitGlobNoMatch;
 			}
 		} else if (ftpcat != 0) {
 #if (defined(WIN32) || defined(_WINDOWS)) && !defined(__CYGWIN__)
@@ -646,6 +683,8 @@ main(int argc, char **argv)
 			} else {
 				FTPPerror(&fi, rc, kErrCouldNotStartDataTransfer, "ncftpget", NULL);
 				es = kExitXferFailed;
+				if (fi.errNo == kErrGlobNoMatch)
+					es = kExitGlobNoMatch;
 			}
 #if (defined(WIN32) || defined(_WINDOWS)) && !defined(__CYGWIN__)
 			_setmode(STDOUT_FILENO, _O_TEXT);
@@ -659,6 +698,8 @@ main(int argc, char **argv)
 				} else {
 					FTPPerror(&fi, rc, kErrCouldNotStartDataTransfer, "ncftpget", NULL);
 					es = kExitXferFailed;
+					if (fi.errNo == kErrGlobNoMatch)
+						es = kExitGlobNoMatch;
 				}
 			} else {
 				es = kExitSuccess;
@@ -674,6 +715,8 @@ main(int argc, char **argv)
 				es = kExitSuccess;
 			else
 				es = kExitXferFailed;
+			if (fi.errNo == kErrGlobNoMatch)
+				es = kExitGlobNoMatch;
 		} else if (ftpcat != 0) {
 #if (defined(WIN32) || defined(_WINDOWS)) && !defined(__CYGWIN__)
 			_setmode(STDOUT_FILENO, _O_BINARY);
@@ -685,11 +728,15 @@ main(int argc, char **argv)
 #if (defined(WIN32) || defined(_WINDOWS)) && !defined(__CYGWIN__)
 			_setmode(STDOUT_FILENO, _O_TEXT);
 #endif
+			if (fi.errNo == kErrGlobNoMatch)
+				es = kExitGlobNoMatch;
 		} else {
 			if (Copy(&fi, dstdir, flist, rflag, xtype, resumeflag, appendflag, deleteflag, tarflag, perfilecmd) == 0)
 				es = kExitSuccess;
 			else
 				es = kExitXferFailed;
+			if (fi.errNo == kErrGlobNoMatch)
+				es = kExitGlobNoMatch;
 		}
 	}
 
