@@ -14,11 +14,13 @@
 
 #if (defined(WIN32) || defined(_WINDOWS)) && !defined(__CYGWIN__)
 #	include "..\ncftp\util.h"
+#	include "..\ncftp\bookmark.h"
 #	include "..\ncftp\spool.h"
 #	include "..\ncftp\pref.h"
 #	include "..\ncftp\gl_getline.h"
 #else
 #	include "../ncftp/util.h"
+#	include "../ncftp/bookmark.h"
 #	include "../ncftp/spool.h"
 #	include "../ncftp/pref.h"
 #	include "../ncftp/gl_getline.h"
@@ -37,6 +39,7 @@ extern unsigned int gFirewallPort;
 extern char gFirewallExceptionList[256];
 extern int gFwDataPortMode;
 extern const char gOS[], gVersion[];
+extern Bookmark gBm;
 
 static int FTPRemoteRecursiveMList(FTPCIPtr cip, const char *const rdir, /* FTPFileInfoListPtr files, */ FTPLineListPtr lines);
 
@@ -70,7 +73,7 @@ FTPRemoteRecursiveMListSubdir(FTPCIPtr cip, char *const parentdir, const size_t
 	}
 
 	(void) AddLine(lines, "");
-	if (Dynscpy(&newl, relpath, ":", 0) != NULL) {
+	if (Dynscpy(&newl, relpath, ":", (char *) 0) != NULL) {
 		(void) AddLine(lines, newl);
 		free(newl);
 	}
@@ -103,7 +106,7 @@ FTPRemoteRecursiveMListSubdir(FTPCIPtr cip, char *const parentdir, const size_t
 				 */
 				newl = NULL;
 				*cp++ = '\0';
-				if (Dynscpy(&newl, lp->line, " ", relpath, "/", cp, 0) != NULL) {
+				if (Dynscpy(&newl, lp->line, " ", relpath, "/", cp, (char *) 0) != NULL) {
 					free(lp->line);
 					lp->line = newl;
 				}
@@ -232,6 +235,7 @@ Usage(void)
 	(void) fprintf(fp, "\
   -t XX  Timeout after XX seconds.\n\
   -f XX  Read the file XX for user and password information.\n\
+         If file XX does not exist, check for bookmark XX in $HOME/.ncftp/bookmarks.\n\
   -E     Use regular (PORT) data connections.\n\
   -F     Use passive (PASV) data connections (default).\n\
   -K     Show disk usage by attempting SITE DF.\n");
@@ -320,6 +324,7 @@ main(int argc, char **argv)
 	FILE *ofp, *savedDebugLog = NULL;
 	char precmd[320], postcmd[320], perfilecmd[320];
 	int do_extracmds = 0;
+	int configLoaded = kConfigNotLoaded;
 	GetoptInfo opt;
 
 	InitWinsock();
@@ -362,34 +367,14 @@ main(int argc, char **argv)
 	es = kExitSuccess;
 
 	GetoptReset(&opt);
-	while ((c = Getopt(&opt, argc, argv, "1lRx:P:u:j:p:h:e:d:t:r:f:o:EFKW:X:Y:maCi:gI:Z")) > 0) switch(c) {
+#define kGetoptOptions "1lRx:P:u:j:p:h:e:d:t:r:f:o:EFKW:X:Y:maCi:gI:Z"
+	while ((c = Getopt(&opt, argc, argv, kGetoptOptions)) > 0) switch(c) {
 		case 'P':
-			fi.port = atoi(opt.arg);	
-			break;
 		case 'u':
-			(void) STRNCPY(fi.user, opt.arg);
-			memset(opt.arg, 0, strlen(fi.user));
-			opt.arg[0] = '?';
-			break;
 		case 'j':
-			(void) STRNCPY(fi.acct, opt.arg);
-			memset(opt.arg, 0, strlen(fi.acct));
-			opt.arg[0] = '?';
-			break;
 		case 'p':
-			(void) STRNCPY(fi.pass, opt.arg);	/* Don't recommend doing this! */
-			if (fi.pass[0] == '\0')
-				fi.passIsEmpty = 1;
-			memset(opt.arg, 0, strlen(fi.pass));
-			opt.arg[0] = '?';
-			break;
 		case 'h':
-			/* Doesn't make sense really, but implement for
-			 * compatibility with the other utilities.
-			 */
-			(void) STRNCPY(fi.host, opt.arg);
-			memset(opt.arg, 0, strlen(fi.user));
-			opt.arg[0] = '?';
+			/* Use these options later */
 			break;
 		case 'e':
 			if (strcmp(opt.arg, "stdout") == 0)
@@ -422,7 +407,10 @@ main(int argc, char **argv)
 			SetRedial(&fi, opt.arg);
 			break;
 		case 'f':
-			ReadConfigFile(opt.arg, &fi);
+			if ((configLoaded = ReadConfigFile(opt.arg, &fi)) == kErrorLoadingConfig) {
+				perror(opt.arg);
+				exit(kExitBadConfigFile);
+			}
 			break;
 		case 'o':
 			fi.manualOverrideFeatures = opt.arg;
@@ -500,6 +488,39 @@ main(int argc, char **argv)
 	if (opt.ind > argc - 1)
 		Usage();
 
+	/* In case we loaded a bookmark or config file, we allow the following
+	 * options to override those settings.
+	 */
+	GetoptReset(&opt);
+	while ((c = Getopt(&opt, argc, argv, kGetoptOptions)) > 0) switch(c) {
+		case 'P':
+			fi.port = atoi(opt.arg);	
+			break;
+		case 'u':
+			(void) STRNCPY(fi.user, opt.arg);
+			memset(opt.arg, 0, strlen(fi.user));
+			opt.arg[0] = '?';
+			break;
+		case 'j':
+			(void) STRNCPY(fi.acct, opt.arg);
+			memset(opt.arg, 0, strlen(fi.acct));
+			opt.arg[0] = '?';
+			break;
+		case 'p':
+			(void) STRNCPY(fi.pass, opt.arg);	/* Don't recommend doing this! */
+			fi.leavePass = 1;
+			if (fi.pass[0] == '\0')
+				fi.passIsEmpty = 1;
+			memset(opt.arg, 0, strlen(fi.pass));
+			opt.arg[0] = '?';
+			break;
+		case 'h':
+			(void) STRNCPY(fi.host, opt.arg);
+			memset(opt.arg, 0, strlen(fi.user));
+			opt.arg[0] = '?';
+			break;
+	}
+
 	STRNCPY(lsflags, "-CF");
 	if (lslong != 0) {
 		STRNCPY(lsflags, "-l");
