@@ -14,11 +14,13 @@
 
 #if (defined(WIN32) || defined(_WINDOWS)) && !defined(__CYGWIN__)
 #	include "..\ncftp\util.h"
+#	include "..\ncftp\bookmark.h"
 #	include "..\ncftp\spool.h"
 #	include "..\ncftp\pref.h"
 #	include "..\ncftp\gl_getline.h"
 #else
 #	include "../ncftp/util.h"
+#	include "../ncftp/bookmark.h"
 #	include "../ncftp/spool.h"
 #	include "../ncftp/pref.h"
 #	include "../ncftp/gl_getline.h"
@@ -37,6 +39,7 @@ extern unsigned int gFirewallPort;
 extern char gFirewallExceptionList[256];
 extern int gFwDataPortMode;
 extern const char gOS[], gVersion[];
+extern Bookmark gBm;
 
 static void
 Usage(void)
@@ -46,7 +49,8 @@ Usage(void)
 	fp = OpenPager();
 	(void) fprintf(fp, "NcFTPPut %.5s\n\n", gVersion + 11);
 	(void) fprintf(fp, "Usages:\n");
-	(void) fprintf(fp, "  ncftpput [flags] remote-host remote-dir local-files...   (mode 1)\n");
+	(void) fprintf(fp, "  ncftpput [flags] remote-host remote-dir local-files...   (mode 1a)\n");
+	(void) fprintf(fp, "  ncftpput [flags] bookmark-name remote-dir local-files... (mode 1b)\n");
 	(void) fprintf(fp, "  ncftpput -f login.cfg [flags] remote-dir local-files...  (mode 2)\n");
 	(void) fprintf(fp, "  ncftpput -c remote-host remote-path-name < stdin         (mode 3)\n");
 	(void) fprintf(fp, "  ncftpput -C remote-host local-path-name remote-path-name (mode 4)\n");
@@ -63,7 +67,8 @@ Usage(void)
   -a     Use ASCII transfer type instead of binary.\n\
   -m     Attempt to mkdir the dstdir before copying.\n\
   -v/-V  Do (do not) use progress meters.\n\
-  -f XX  Read the file XX for host, user, and password information.\n");
+  -f XX  Read the file XX for host, user, and password information.\n\
+         If file XX does not exist, check for bookmark XX in $HOME/.ncftp/bookmarks.\n");
 	(void) fprintf(fp, "\
   -h XX  Connect to host XX.  Useful for overriding host in -f config.file.\n\
   -c     Read locally from stdin and write remotely to specified pathname.\n\
@@ -193,6 +198,7 @@ main(int argc, char **argv)
 	const char *udirp;
 	char ufile[256];
 	char precmd[320], postcmd[320], perfilecmd[320];
+	int configLoaded = kConfigNotLoaded, configLoaded2 = kConfigNotLoaded;
 	GetoptInfo opt;
 
 	InitWinsock();
@@ -254,30 +260,11 @@ main(int argc, char **argv)
 	GetoptReset(&opt);
 	while ((c = Getopt(&opt, argc, argv, "P:u:j:p:h:e:d:U:t:mar:RvVf:o:AT:S:EFcCyZzDbB:W:X:Y:I:")) > 0) switch(c) {
 		case 'P':
-			fi.port = atoi(opt.arg);	
-			break;
 		case 'u':
-			(void) STRNCPY(fi.user, opt.arg);
-			memset(opt.arg, 0, strlen(fi.user));
-			opt.arg[0] = '?';
-			break;
 		case 'j':
-			(void) STRNCPY(fi.acct, opt.arg);
-			memset(opt.arg, 0, strlen(fi.acct));
-			opt.arg[0] = '?';
-			break;
 		case 'p':
-			(void) STRNCPY(fi.pass, opt.arg);	/* Don't recommend doing this! */
-			fi.leavePass = 1;
-			if (fi.pass[0] == '\0')
-				fi.passIsEmpty = 1;
-			memset(opt.arg, 0, strlen(fi.pass));
-			opt.arg[0] = '?';
-			break;
 		case 'h':
-			(void) STRNCPY(fi.host, opt.arg);
-			memset(opt.arg, 0, strlen(fi.user));
-			opt.arg[0] = '?';
+			/* Use these options later */
 			break;
 		case 'e':
 			if (strcmp(opt.arg, "stdout") == 0)
@@ -333,7 +320,10 @@ main(int argc, char **argv)
 			progmeters = 0;
 			break;
 		case 'f':
-			ReadConfigFile(opt.arg, &fi);
+			if ((configLoaded = ReadConfigFile(opt.arg, &fi)) == kErrorLoadingConfig) {
+				perror(opt.arg);
+				exit(kExitBadConfigFile);
+			}
 			usingcfg = 1;
 			break;
 		case 'o':
@@ -458,6 +448,44 @@ main(int argc, char **argv)
 			(void) STRNCPY(fi.host, argv[opt.ind]);
 			dstfile = argv[opt.ind + 1];
 		}
+		if (strchr(fi.host, '.') == NULL) {
+			/* Check if host argument corresponds to config file or entry from ~/.ncftp/bookmarks. */
+
+			configLoaded2 = ReadConfigFile(fi.host, &fi);
+		}
+	}
+
+	/* In case we loaded a bookmark or config file, we allow the following
+	 * options to override those settings.
+	 */
+	GetoptReset(&opt);
+	while ((c = Getopt(&opt, argc, argv, "P:u:j:p:h:e:d:t:aRTr:vVf:o:ADzZEFbcCB:W:X:Y:I:")) > 0) switch(c) {
+		case 'P':
+			fi.port = atoi(opt.arg);	
+			break;
+		case 'u':
+			(void) STRNCPY(fi.user, opt.arg);
+			memset(opt.arg, 0, strlen(fi.user));
+			opt.arg[0] = '?';
+			break;
+		case 'j':
+			(void) STRNCPY(fi.acct, opt.arg);
+			memset(opt.arg, 0, strlen(fi.acct));
+			opt.arg[0] = '?';
+			break;
+		case 'p':
+			(void) STRNCPY(fi.pass, opt.arg);	/* Don't recommend doing this! */
+			fi.leavePass = 1;
+			if (fi.pass[0] == '\0')
+				fi.passIsEmpty = 1;
+			memset(opt.arg, 0, strlen(fi.pass));
+			opt.arg[0] = '?';
+			break;
+		case 'h':
+			(void) STRNCPY(fi.host, opt.arg);
+			memset(opt.arg, 0, strlen(fi.user));
+			opt.arg[0] = '?';
+			break;
 	}
 
 	if (strcmp(fi.user, "anonymous") && strcmp(fi.user, "ftp")) {
@@ -567,6 +595,16 @@ main(int argc, char **argv)
 
 	(void) AdditionalCmd(&fi, precmd, NULL);
 
+	if ((configLoaded2 == kLoadedFromBookmark) && (strcmp(gBm.dir, "/") != 0) && (strcmp(gBm.dir, ".") != 0)) {
+		if ((dstdir != NULL) && ((dstdir[0] == '/') || (dstdir[0] == '\\'))) {
+			/* Don't bother if we are just going to change to an absolute path next */
+			result = kNoErr;
+		} else if ((result = FTPChdir(&fi, gBm.dir)) < 0) {
+			FTPPerror(&fi, result, kErrCWDFailed, "Warning: Could not change to bookmark's remote directory of", gBm.dir);
+		}
+
+	}
+
 	if (dstdir != NULL) {
 		es = kExitChdirTimedOut;
 		if (wantMkdir != 0) {
