@@ -4,7 +4,7 @@
 #endif
 
 static const char UNUSED(gSioVersion[]) = kSioVersion;
-static const char UNUSED(gCopyright[]) = "@(#) libsio Copyright 1992-2003, by Mike Gleason.  All rights reserved.";
+static const char UNUSED(gCopyright[]) = "@(#) libsio Copyright 1992-2005, by Mike Gleason.  All rights reserved.";
 
 
 /* Read up to "size" bytes on sfd before "tlen" seconds.
@@ -74,7 +74,7 @@ SRead(int sfd, char *const buf0, size_t size, int tlen, int retry)
 				tv.tv_sec = (tv_sec_t) tleft;
 				tv.tv_usec = 0;
 				result = select(sfd + 1, SELECT_TYPE_ARG234 &ss, NULL, NULL, SELECT_TYPE_ARG5 &tv);
-				if (result == 1) {
+				if (result >= 1) {
 					/* ready */
 					break;
 				} else if (result == 0) {
